import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';

class MyButton extends StatelessWidget {
  final double width;
  final double height;
  final Color color;
  final Widget child;
  final Function onPress;
  final bool isLoading;
  final double borderRadius;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  const MyButton(
      {Key key,
      this.isLoading = false,
      this.borderRadius = 12,
      this.color = Colors.deepOrange,
      this.width,
      this.height = 50,
      @required this.onPress,
      @required this.child,
      this.margin,
      this.padding})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      padding: padding,
      child: SizedBox(
        width: width,
        height: height,
        child: RaisedButton(
          padding: EdgeInsets.all(0),
          color: color,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(borderRadius),
          ),
          onPressed: onPress,
          child: isLoading
              ? Center(
                  child: SpinKitCircle(
                    color: Colors.white,
                    size: height * 0.8,
                  ),
                )
              : child,
        ),
      ),
    );
  }
}

class MyButtonGradient extends StatelessWidget {
  final double width;
  final double height;
  final Widget child;
  final Function onPress;
  final bool isLoading;
  final double borderRadius;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final Gradient gradient;

  const MyButtonGradient(
      {Key key,
      this.isLoading = false,
      this.borderRadius = 12,
      this.width,
      this.height = 50,
      @required this.onPress,
      @required this.child,
      this.margin,
      this.padding,
      this.gradient})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: padding,
      margin: margin,
      width: width,
      height: height,
      decoration: BoxDecoration(gradient: gradient, borderRadius: BorderRadius.circular(borderRadius), boxShadow: [
        BoxShadow(
          color: Colors.grey[500],
          offset: Offset(0.0, 1.5),
          blurRadius: 1.5,
        ),
      ]),
      child: Material(
        color: Colors.transparent,
        child: isLoading
            ? Center(
                child: SpinKitCircle(
                color: Colors.white,
                size: height * 0.8,
              ))
            : InkWell(
                onTap: onPress,
                child: Center(
                  child: child,
                )),
      ),
    );
  }
}

class MyFlatButton extends StatelessWidget {
  final double width;
  final double height;
  final Widget child;
  final Function onPress;
  final bool isLoading;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  const MyFlatButton({
    Key key,
    this.isLoading = false,
    this.width,
    this.height,
    @required this.onPress,
    @required this.child,
    this.margin,
    this.padding,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      width: width,
      height: height,
      margin: margin,
      padding: padding,
      child: Material(
        child: isLoading
            ? SpinKitCircle(
                color: Colors.white,
                size: height * 0.8,
              )
            : InkWell(onTap: onPress, child: child),
        color: Colors.transparent,
      ),
    );
  }
}
